@@ -1,7 +1,6 @@
 // Copyright 2018-2021 the Deno authors. All rights reserved. MIT license.
 
 use deno_core::error::not_supported;
-use deno_core::error::null_opbuf;
 use deno_core::error::resource_unavailable;
 use deno_core::error::AnyError;
 use deno_core::op_async;
@@ -17,8 +16,8 @@ use deno_core::Resource;
 use deno_core::ResourceId;
 use deno_core::ZeroCopyBuf;
 use deno_net::io::TcpStreamResource;
-use deno_net::io::TlsStreamResource;
 use deno_net::io::UnixStreamResource;
+use deno_net::ops_tls::TlsStreamResource;
 use std::borrow::Cow;
 use std::cell::RefCell;
 use std::io::Read;
@@ -387,9 +386,8 @@ impl Resource for StdFileResource {
 fn op_read_sync(
   state: &mut OpState,
   rid: ResourceId,
-  buf: Option<ZeroCopyBuf>,
+  mut buf: ZeroCopyBuf,
 ) -> Result<u32, AnyError> {
-  let mut buf = buf.ok_or_else(null_opbuf)?;
   StdFileResource::with(state, rid, move |r| match r {
     Ok(std_file) => std_file
       .read(&mut buf)
@@ -402,22 +400,21 @@ fn op_read_sync(
 async fn op_read_async(
   state: Rc<RefCell<OpState>>,
   rid: ResourceId,
-  buf: Option<ZeroCopyBuf>,
+  mut buf: ZeroCopyBuf,
 ) -> Result<u32, AnyError> {
-  let buf = &mut buf.ok_or_else(null_opbuf)?;
   let resource = state.borrow().resource_table.get_any(rid)?;
   let nread = if let Some(s) = resource.downcast_rc::<ChildStdoutResource>() {
-    s.read(buf).await?
+    s.read(&mut buf).await?
   } else if let Some(s) = resource.downcast_rc::<ChildStderrResource>() {
-    s.read(buf).await?
+    s.read(&mut buf).await?
   } else if let Some(s) = resource.downcast_rc::<TcpStreamResource>() {
-    s.read(buf).await?
+    s.read(&mut buf).await?
   } else if let Some(s) = resource.downcast_rc::<TlsStreamResource>() {
-    s.read(buf).await?
+    s.read(&mut buf).await?
   } else if let Some(s) = resource.downcast_rc::<UnixStreamResource>() {
-    s.read(buf).await?
+    s.read(&mut buf).await?
   } else if let Some(s) = resource.downcast_rc::<StdFileResource>() {
-    s.read(buf).await?
+    s.read(&mut buf).await?
   } else {
     return Err(not_supported());
   };
@@ -427,9 +424,8 @@ async fn op_read_async(
 fn op_write_sync(
   state: &mut OpState,
   rid: ResourceId,
-  buf: Option<ZeroCopyBuf>,
+  buf: ZeroCopyBuf,
 ) -> Result<u32, AnyError> {
-  let buf = buf.ok_or_else(null_opbuf)?;
   StdFileResource::with(state, rid, move |r| match r {
     Ok(std_file) => std_file
       .write(&buf)
@@ -442,20 +438,19 @@ fn op_write_sync(
 async fn op_write_async(
   state: Rc<RefCell<OpState>>,
   rid: ResourceId,
-  buf: Option<ZeroCopyBuf>,
+  buf: ZeroCopyBuf,
 ) -> Result<u32, AnyError> {
-  let buf = &buf.ok_or_else(null_opbuf)?;
   let resource = state.borrow().resource_table.get_any(rid)?;
   let nwritten = if let Some(s) = resource.downcast_rc::<ChildStdinResource>() {
-    s.write(buf).await?
+    s.write(&buf).await?
   } else if let Some(s) = resource.downcast_rc::<TcpStreamResource>() {
-    s.write(buf).await?
+    s.write(&buf).await?
   } else if let Some(s) = resource.downcast_rc::<TlsStreamResource>() {
-    s.write(buf).await?
+    s.write(&buf).await?
   } else if let Some(s) = resource.downcast_rc::<UnixStreamResource>() {
-    s.write(buf).await?
+    s.write(&buf).await?
   } else if let Some(s) = resource.downcast_rc::<StdFileResource>() {
-    s.write(buf).await?
+    s.write(&buf).await?
   } else {
     return Err(not_supported());
   };
